@@ -1,11 +1,10 @@
-# mysql/json.py
-# Copyright (C) 2005-2022 the SQLAlchemy authors and contributors
+# dialects/mysql/json.py
+# Copyright (C) 2005-2024 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: https://www.opensource.org/licenses/mit-license.php
-
-from __future__ import absolute_import
+# mypy: ignore-errors
 
 from ... import types as sqltypes
 
@@ -29,14 +28,12 @@ class JSON(sqltypes.JSON):
     datatype, by adapting the operations to render the ``JSON_EXTRACT``
     function at the database level.
 
-    .. versionadded:: 1.1
-
     """
 
     pass
 
 
-class _FormatTypeMixin(object):
+class _FormatTypeMixin:
     def _format_value(self, value):
         raise NotImplementedError()
 
